<?php
namespace Ak\Test;

use PHPUnit\DbUnit\Database\Connection;
use PHPUnit\Framework\TestCase;
use Psr\Container\ContainerInterface;
use Laminas\ConfigAggregator\ArrayProvider;
use Laminas\ConfigAggregator\ConfigAggregator;
use Laminas\ServiceManager\ServiceManager;
use Laminas\Stdlib\ArrayUtils;

/**
 * Osnovna klasa za testove.
 *
 * @package Ak\Test
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/test
 */
abstract class AbstractTestCase extends TestCase
{
    use DOMTestTrait;

    /**
     * ConfigProvideri komponenti koje ovaj test koristi.
     *
     * @var string[]
     */
    public $configProviders = [];

    /**
     * Zadnji generirani container.
     *
     * @var ServiceManager
     */
    public $lastContainer;

    /**
     * Konekcije koje ovaj test koristi.
     *
     * @var Connection[]
     */
    public $dbConnections = [];

    //<editor-fold desc="1. Konfiguracija">
    /**
     * Vraća konfiguraciju sakupljenu iz sljedećih izvora:
     *
     * 1. ConfigProvideri postavljeni u $this->configProviders.
     * 2. Konfiguraciju poslanu kao argument metode.
     *
     * @param array $config
     *
     * @return array
     */
    public function getConfig(array $config = []): array
    {
        return (new ConfigAggregator(ArrayUtils::merge($this->configProviders, [
            new ArrayProvider($config),
        ])))->getMergedConfig();
    }

    /**
     * Utvrđuje da li dati konfiguracijski array sadrži dati ključ.
     *
     * @param string $key
     * @param array $config
     * @param string $message
     */
    public static function assertConfigHasKey(string $key, array $config, string $message = ''): void
    {
        $constraint = new Constraint\ConfigHasKey($key);
        static::assertThat($config, $constraint, $message);
    }

    /**
     * Utvrđuje da li dati konfiguracijski array pod datim ključem ima očekivanu vrijednost.
     *
     * @param string $key
     * @param $expected
     * @param array $config
     * @param string $message
     */
    public static function assertConfigValueEquals(string $key, $expected, array $config, string $message = ''): void
    {
        static::assertConfigHasKey($key, $config);

        $constraint = new Constraint\ConfigValueEquals($key, $expected);
        static::assertThat($config, $constraint, $message);
    }
    //</editor-fold>
    //<editor-fold desc="2. Container">
    /**
     * Vraća konfigurirani container. Svaki poziv vraća novu instancu.
     *
     * @param array $config
     *
     * @param array $services
     * @return ServiceManager
     */
    public function getContainer(array $config = [], array $services = []): ServiceManager
    {
        $config = $this->getConfig($config);

        if (isset($config['dependencies']['services']) && is_array($config['dependencies']['services'])) {
            $config['dependencies']['services'] = array_merge($config['dependencies']['services'], $services);
        } else {
            $config['dependencies']['services'] = $services;
        }

        $this->lastContainer = new ServiceManager($config['dependencies'] ?? []);
        $this->lastContainer->setService('config', $config);

        return $this->lastContainer;
    }

    /**
     * Utvrđuje da dati container sadrži servis s datim imenom.
     *
     * @param ContainerInterface $container
     * @param string|array $nameOrNames
     * @param string|null $instanceOf
     * @param bool $shouldBeShared
     * @param string $message
     */
    public static function assertContainerHasService(
        ContainerInterface $container,
        $nameOrNames,
        string $instanceOf = null,
        bool $shouldBeShared = false,
        $message = ''
    ) {
        $constraint = new Constraint\ContainerHasService($nameOrNames);
        static::assertThat($container, $constraint, $message);

        if (null !== $instanceOf) {
            $constraint = new Constraint\ContainerHasInstanceOf($nameOrNames, $instanceOf);
            static::assertThat($container, $constraint, $message);
        }

        if ($shouldBeShared && is_array($nameOrNames)) {
            $constraint = new Constraint\ContainerHasSharedService($nameOrNames);
            static::assertThat($container, $constraint, $message);
        }
    }
    //</editor-fold>

}
