<?php
namespace Ak\Test;

use Ak\Db\Adapter\Driver\Dbal\Connection as DbalConnection;
use Doctrine\DBAL\Schema\AbstractSchemaManager;
use Doctrine\DBAL\Schema\Schema;
use PHPUnit\DbUnit\Database\Connection;
use PHPUnit\DbUnit\Database\DataSet;
use PHPUnit\DbUnit\DataSet\IDataSet;
use PHPUnit\DbUnit\Operation\Operation;
use PHPUnit\DbUnit\TestCaseTrait;
use PHPUnit\DbUnit\Tester;
use Laminas\Db\Adapter\AdapterInterface;
use Laminas\Db\Adapter\Driver\DriverInterface;
use Laminas\ServiceManager\ServiceManager;

/**
 * Pruža alate za testiranje baze.
 *
 * @package Ak\Test
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/mailchimp
 */
trait DbTestTrait
{
    use TestCaseTrait;

    /** @var array */
    protected $dbalConnections = [];

    /**
     * Performs operation returned by getSetUpOperation().
     */
    protected function setUp(): void
    {
        parent::setUp();

        $this->databaseTester = null;

        if ($this->getConnection()) {
            /** @var Db\DbTester $tester */
            $tester = $this->getDatabaseTester();
            $tester->setSetUpOperation($this->getSetUpOperation());
            $tester->setDataSet($this->getDataSet());

            $dbalConnection = $this->getDbalConnection();
            $tester->setDbalConnection($dbalConnection);

            $schema = $dbalConnection->getSchemaManager()->createSchema();
            $this->createSchema($schema);
            $tester->setDbalSchema($schema);

            $tester->onSetUp();
        }
    }

    /**
     * Performs operation returned by getTearDownOperation().
     */
    protected function tearDown(): void
    {
        parent::tearDown();

        if ($this->getConnection()) {
            $this->getDatabaseTester()->setTearDownOperation($this->getTearDownOperation());
            $this->getDatabaseTester()->setDataSet($this->getDataSet());
            $this->getDatabaseTester()->onTearDown();
        }

        /*
         * Destroy the tester after the test is run to keep DB connections
         * from piling up.
         */
        $this->databaseTester = null;
    }

    /**
     * @return null|DbalConnection
     */
    public function getDbalConnection(): ?DbalConnection
    {
        /** @var ServiceManager $container */
        $container = $this->lastContainer ?? $this->getContainer();
        $connectionId = md5(serialize($container->get('config')['db'] ?? []));

        if (!isset($this->dbalConnections[$connectionId])) {
            /** @var AdapterInterface $adapter */
            $adapter = $container->get('db');
            $driver = $adapter->getDriver();

            if ($driver instanceof DriverInterface) {
                $connection = $driver->getConnection();
                $connection->connect();

                $this->dbalConnections[$connectionId] = $connection;
            }
        }

        return $this->dbalConnections[$connectionId] ?? null;
    }

    /**
     * Vraća konekciju na bazu.
     *
     * @return Connection
     */
    public function getConnection()
    {
        if ($dbalConnection = $this->getDbalConnection()) {
            $pdo = $dbalConnection->getResource();

            if (!$pdo) {
                throw new \RuntimeException('Ne mogu izvući PDO objekt.');
            }

            return $this->createDefaultDBConnection($pdo);
        }

        return null;
    }

    /**
     * Returns the test dataset.
     *
     * @return IDataSet
     */
    protected function getDataSet()
    {
        return new DataSet($this->getConnection());
    }

    /**
     * @return AbstractSchemaManager
     */
    public function getSchemaManager(): AbstractSchemaManager
    {
        $connection = $this->getDbalConnection();
        return $connection->getSchemaManager();
    }

    /**
     * Ova metoda može dodati objekte u schemu prije kreiranja.
     *
     * @param Schema $schema
     */
    public function createSchema(Schema $schema): void
    { }

    /**
     * Returns the database operation executed in test setup.
     *
     * @return Operation
     */
    protected function getSetUpOperation()
    {
        return Db\Operation\Factory::CREATE_CLEAN_INSERT();
    }

    /**
     * Creates a IDatabaseTester for this testCase.
     *
     * @return Tester
     */
    protected function newDatabaseTester()
    {
        return new Db\DbTester($this->getConnection());
    }
}
