<?php
namespace AkTest\Test;

use Ak\Test\AbstractTestCase;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;
use Psr\Container\ContainerInterface;
use Laminas\View\Renderer\RendererInterface;

/**
 * Class AbstractTestCaseTest
 *
 * @package AkTest\Test
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/test
 */
class ContainerTest extends TestCase
{
    /**
     * @throws \ReflectionException
     */
    public function testCanCreateContainer()
    {
        /** @var AbstractTestCase|MockObject $test */
        $test = $this->getMockForAbstractClass(AbstractTestCase::class);
        $test->configProviders = [
            TestConfigProvider::class,
        ];

        $container = $test->getContainer();

        $this->assertInstanceOf(ContainerInterface::class, $container);
        $this->assertTrue($container->has('config'));
        $this->assertInternalType('array', $container->get('config'));
    }

    /**
     * @throws \ReflectionException
     */
    public function testCanInjectServices()
    {
        /** @var AbstractTestCase|MockObject $test */
        $test = $this->getMockForAbstractClass(AbstractTestCase::class);
        $service = $this->createMock(AbstractTestCase::class);

        $container = $test->getContainer([
            'dependencies' => [
                'services' => [
                    'some_service' => $service,
                ],
            ],
        ]);

        $this->assertTrue($container->has('some_service'));
        $this->assertSame($service, $container->get('some_service'));
    }

    /**
     * @expectedException \PHPUnit\Framework\AssertionFailedError
     * @throws \ReflectionException
     */
    public function testContainerHasServiceWillFail()
    {
        /** @var AbstractTestCase|MockObject $test */
        $test = $this->getMockForAbstractClass(AbstractTestCase::class);

        $container = $test->getContainer();

        $test->assertContainerHasService($container, 'invalid');
    }

    /**
     * @throws \ReflectionException
     */
    public function testContainerHasServiceWorks()
    {
        /** @var AbstractTestCase|MockObject $test */
        $test = $this->getMockForAbstractClass(AbstractTestCase::class);
        $service = $this->createMock(AbstractTestCase::class);

        $container = $test->getContainer([
            'dependencies' => [
                'services' => [
                    'service' => $service,
                ],
            ],
        ]);

        $test->assertContainerHasService($container, 'service');
    }

    /**
     * @expectedException \PHPUnit\Framework\AssertionFailedError
     * @throws \ReflectionException
     */
    public function testContainerHasServiceWillFailWhenOneOfTheNamesDoesntExist()
    {
        /** @var AbstractTestCase|MockObject $test */
        $test = $this->getMockForAbstractClass(AbstractTestCase::class);
        $service = $this->createMock(AbstractTestCase::class);

        $container = $test->getContainer([
            'dependencies' => [
                'aliases' => [
                    'alias_1' => 'service',
                ],

                'services' => [
                    'service' => $service,
                ],
            ],
        ]);

        $test->assertContainerHasService($container, [
            'alias_1',
            'alias_2',
            'service',
        ]);
    }

    /**
     * @throws \ReflectionException
     */
    public function testContainerHasServiceWorksWithAliases()
    {
        /** @var AbstractTestCase|MockObject $test */
        $test = $this->getMockForAbstractClass(AbstractTestCase::class);
        $service = $this->createMock(AbstractTestCase::class);

        $container = $test->getContainer([
            'dependencies' => [
                'aliases' => [
                    'alias_1' => 'service',
                ],

                'services' => [
                    'service' => $service,
                ],
            ],
        ]);

        $test->assertContainerHasService($container, [
            'alias_1',
            'service',
        ]);
    }

    /**
     * @expectedException \PHPUnit\Framework\AssertionFailedError
     * @throws \ReflectionException
     */
    public function testContainerHasServiceWillFailForInstanceCheck()
    {
        /** @var AbstractTestCase|MockObject $test */
        $test = $this->getMockForAbstractClass(AbstractTestCase::class);
        $service = $this->createMock(AbstractTestCase::class);

        $container = $test->getContainer([
            'dependencies' => [
                'services' => [
                    'service' => $service,
                ],
            ],
        ]);

        $test->assertContainerHasService($container, 'service', RendererInterface::class);
    }

    /**
     * @throws \ReflectionException
     */
    public function testContainerHasServiceWillCheckInstanceOf()
    {
        /** @var AbstractTestCase|MockObject $test */
        $test = $this->getMockForAbstractClass(AbstractTestCase::class);
        $service = $this->createMock(RendererInterface::class);

        $container = $test->getContainer([
            'dependencies' => [
                'services' => [
                    'service' => $service,
                ],
            ],
        ]);

        $test->assertContainerHasService($container, 'service', RendererInterface::class);
    }

    /**
     * @expectedException \PHPUnit\Framework\AssertionFailedError
     * @throws \ReflectionException
     */
    public function testContainerHasServiceWillFailWhenServiceNotShared()
    {
        /** @var AbstractTestCase|MockObject $test */
        $test = $this->getMockForAbstractClass(AbstractTestCase::class);
        $service = $this->createMock(RendererInterface::class);
        $service2 = $this->createMock(RendererInterface::class);

        $container = $test->getContainer([
            'dependencies' => [
                'aliases' => [
                    'alias_1' => 'service',
                    'alias_2' => 'service2',
                ],

                'services' => [
                    'service' => $service,
                    'service2' => $service2,
                ],
            ],
        ]);

        $test->assertContainerHasService($container, ['service', 'alias_1', 'alias_2'], RendererInterface::class, true);
    }

    /**
     * @throws \ReflectionException
     */
    public function testContainerHasServiceCheckForSharedService()
    {
        /** @var AbstractTestCase|MockObject $test */
        $test = $this->getMockForAbstractClass(AbstractTestCase::class);
        $service = $this->createMock(RendererInterface::class);

        $container = $test->getContainer([
            'dependencies' => [
                'aliases' => [
                    'alias_1' => 'service',
                    'alias_2' => 'service',
                ],

                'services' => [
                    'service' => $service,
                ],
            ],
        ]);

        $test->assertContainerHasService($container, ['service', 'alias_1', 'alias_2'], RendererInterface::class, true);
    }

    /**
     * @throws \ReflectionException
     */
    public function testWillSaveLastCreatedContainer()
    {
        /** @var AbstractTestCase|MockObject $test */
        $test = $this->getMockForAbstractClass(AbstractTestCase::class);

        $container = $test->getContainer();
        $this->assertSame($container, $test->lastContainer);
    }
}
