<?php
namespace AkTest\Test;

use Ak\Db\Adapter\Driver\Dbal\Connection as DbalConnection;
use Ak\Db\Adapter\Driver\Dbal\Dbal;
use Ak\Db\ConfigProvider as DbConfigProvider;
use Doctrine\DBAL\Schema\Table;
use PHPUnit\DbUnit\Database\Connection;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;
use Laminas\Db\Adapter\AdapterInterface;

/**
 * Class DbTest
 *
 * @package AkTest\Test
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/test
 */
class DbTest extends TestCase
{
    /**
     * @throws \ReflectionException
     */
    public function testCanCreateConnectionFromAdapter()
    {
        /** @var MockDbTestCase|MockObject $test */
        $test = $this->getMockForAbstractClass(MockDbTestCase::class);
        $test->configProviders = [
            DbConfigProvider::class,
        ];

        $test->getContainer([
            'db' => [
                'driver' => 'db.driver.sqlite',
                'database' => ':memory:',
            ],
        ]);

        $connection = $test->getConnection();
        $this->assertInstanceOf(Connection::class, $connection);
    }

    /**
     * @expectedException \RuntimeException
     * @throws \ReflectionException
     */
    public function testWillThrowExceptionIfItCannotExtractThePdoObject()
    {
        /** @var MockDbTestCase|MockObject $test */
        $test = $this->getMockForAbstractClass(MockDbTestCase::class);
        $test->configProviders = [
            DbConfigProvider::class,
        ];

        $connection = $this->createMock(DbalConnection::class);

        $driver = $this->createMock(Dbal::class);
        $driver->expects($this->once())
            ->method('getConnection')
            ->willReturn($connection);

        $adapter = $this->createMock(AdapterInterface::class);
        $adapter->expects($this->once())
            ->method('getDriver')
            ->willReturn($driver);

        $test->getContainer([
            'dependencies' => [
                'aliases' => [
                    'db' => AdapterInterface::class,
                ],

                'services' => [
                    AdapterInterface::class => $adapter,
                ],
            ],
        ]);

        $test->getConnection();
    }

    public function testCanCreateTheSchema()
    {
        $test = new MockDbTestCase();

        $test->runSetUp();
        $schemaManager = $test->getSchemaManager();

        /** @var Table[]|array $tables */
        $tables = $schemaManager->listTables();

        $this->assertCount(1, $tables);
        $this->assertInstanceOf(Table::class, $tables[0]);
        $this->assertEquals('some_table', $tables[0]->getName());
    }

    /**
     * @throws \Doctrine\DBAL\DBALException
     */
    public function testCanPopulateTheDatabase()
    {
        $test = new MockDbTestCase();

        $test->runSetUp();
        $conn = $test->getDbalConnection();

        $stmt = $conn->query('SELECT * FROM some_table');
        $result = $stmt->fetchAll();

        $this->assertCount(2, $result);
        $this->assertEquals(1, $result[0]['id']);
        $this->assertEquals('One', $result[0]['name']);
        $this->assertEquals(2, $result[1]['id']);
        $this->assertEquals('Two', $result[1]['name']);
    }
}
